Imports System.Xml.Serialization
Imports System.IO

Public Class Form1
    ' A dot object.
    <Serializable()> _
    Public Class Dot
        Public X As Integer
        Public Y As Integer
        Public Radius As Integer

        Public Sub New()
        End Sub
        Public Sub New(ByVal new_x As Integer, ByVal new_y As Integer, ByVal new_radius As Integer)
            X = new_x
            Y = new_y
            Radius = new_radius
        End Sub

        Public Sub Draw(ByVal gr As Graphics)
            gr.DrawEllipse(Pens.Black, X - Radius, Y - Radius, 2 * Radius, 2 * Radius)
        End Sub
    End Class

    ' Snapshots for undo/redo.
    Private m_Snapshots As New Collection
    Private m_CurrentSnapshot As Integer = 0
    Private Const MAX_SNAPSHOTS As Integer = 5

    ' The current dots.
    Private m_NumDots As Integer = 0
    Private m_Dots() As Dot

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' Save an empty snapshot.
        SaveSnapshot()
    End Sub

    ' Make a new dot.
    Private Sub Form1_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        MakeDot(e.X, e.Y)

        Me.Invalidate()
    End Sub

    ' Make a dot.
    Private Sub MakeDot(ByVal X As Integer, ByVal Y As Integer)
        Dim rand As New Random
        Dim new_dot As New Dot(X, Y, rand.Next(10, 30))

        m_NumDots += 1
        ReDim Preserve m_Dots(0 To m_NumDots - 1)
        m_Dots(m_NumDots - 1) = new_dot

        ' Save a snapshot.
        SaveSnapshot()
    End Sub

    ' Save a snapshot.
    Private Sub SaveSnapshot()
        ' Remove any snapshots after the current one.
        Do While m_Snapshots.Count > m_CurrentSnapshot
            m_Snapshots.Remove(m_Snapshots.Count)
        Loop

        Dim xml_serializer As New XmlSerializer(GetType(Dot()))
        Dim string_writer As New StringWriter()
        xml_serializer.Serialize(string_writer, m_Dots)
        m_Snapshots.Add(string_writer.ToString())

        string_writer.Close()
        string_writer.Dispose()

        ' Make sure we don't have too many.
        Do While m_Snapshots.Count > MAX_SNAPSHOTS + 1
            m_Snapshots.Remove(1)
        Loop

        m_CurrentSnapshot = m_Snapshots.Count
    End Sub

    ' Restore a snapshot.
    Private Sub RestoreSnapshot()
        Dim xml_serializer As New XmlSerializer(GetType(Dot()))
        Dim string_reader As New StringReader(CStr(m_Snapshots.Item(m_CurrentSnapshot)))
        m_Dots = DirectCast(xml_serializer.Deserialize(string_reader), Dot())
        If m_Dots Is Nothing Then
            m_NumDots = 0
        Else
            m_NumDots = m_Dots.Length
        End If
        string_reader.Close()
        string_reader.Dispose()

        Me.Invalidate()
    End Sub

    ' Draw the dots.
    Private Sub Form1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        e.Graphics.Clear(Me.BackColor)
        For i As Integer = 0 To m_NumDots - 1
            m_Dots(i).Draw(e.Graphics)
        Next i
    End Sub

    Private Sub mnuDataUndo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuDataUndo.Click
        If m_CurrentSnapshot > 1 Then
            m_CurrentSnapshot -= 1
            RestoreSnapshot()
        End If
    End Sub

    Private Sub mnuDataRedo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuDataRedo.Click
        If m_CurrentSnapshot < m_Snapshots.Count Then
            m_CurrentSnapshot += 1
            RestoreSnapshot()
        End If
    End Sub
End Class
