Imports System.Math
Imports System.Collections.Generic

Public Class Form1
    Private m_Handlers As New List(Of NumberHandler)

    ' Make the chain of command.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        m_Handlers.Add(New PrimeHandler())
        m_Handlers.Add(New PowerHandler())
        m_Handlers.Add(New EvenHandler())
        m_Handlers.Add(New UnhandledHandler())
    End Sub

    ' Process a number.
    Private Sub btnProcess_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnProcess.Click
        Dim value As Integer = Integer.Parse(txtValue.Text)

        For Each handler As NumberHandler In m_Handlers
            If handler.WasHandled(value) Then Exit For
        Next handler
    End Sub
End Class

Public MustInherit Class NumberHandler
    Public MustOverride Function WasHandled(ByVal value As Integer) As Boolean
End Class

Public Class PrimeHandler
    Inherits NumberHandler

    ' Return True if we handle this item.
    Public Overrides Function WasHandled(ByVal value As Integer) As Boolean
        Dim is_prime As Boolean = False
        If value < 0 Then value = -value
        If value <= 1 Then
            is_prime = False    ' 0 and 1 are not prime.
        ElseIf value <= 2 Then
            is_prime = True     ' 2 is prime.
        ElseIf value Mod 2 = 0 Then
            is_prime = False    ' Divisible by 2.
        Else
            is_prime = True
            For i As Integer = 3 To CInt(Sqrt(value)) Step 2
                ' See if value is divisible by i.
                If value Mod i = 0 Then
                    is_prime = False
                    Exit For
                End If
            Next i
        End If

        ' See if it's a prime.
        If is_prime Then
            ' Prime.
            MessageBox.Show(value.ToString() & " is prime", _
                "Prime", MessageBoxButtons.OK, MessageBoxIcon.Information)
            Return True
        Else
            ' Not prime.
            Return False
        End If
    End Function
End Class

Public Class PowerHandler
    Inherits NumberHandler

    ' Return True if we handle this item.
    Public Overrides Function WasHandled(ByVal value As Integer) As Boolean
        If value < 0 Then value = -value
        If value < 4 Then Return False

        For base As Integer = 2 To CInt(Sqrt(value))
            Dim power As Double = Log(value, base)
            If CInt(power) = power Then
                ' value = base ^ power.
                MessageBox.Show(value.ToString() & " = " & base.ToString() & "^" & power.ToString(), _
                    "Power", MessageBoxButtons.OK, MessageBoxIcon.Information)
                Return True
            End If
        Next base

        ' Not a power.
        Return False
    End Function
End Class

Public Class EvenHandler
    Inherits NumberHandler

    ' Return True if we handle this item.
    Public Overrides Function WasHandled(ByVal value As Integer) As Boolean
        If value Mod 2 = 0 Then
            ' Even.
            MessageBox.Show(value.ToString() & " is even", _
                "Even", MessageBoxButtons.OK, MessageBoxIcon.Information)
            Return True
        Else
            Return False
        End If
    End Function
End Class

Public Class UnhandledHandler
    Inherits NumberHandler

    ' Always handle the item.
    Public Overrides Function WasHandled(ByVal value As Integer) As Boolean
        MessageBox.Show(value.ToString() & " was not handled", _
            "Unhandled", MessageBoxButtons.OK, MessageBoxIcon.Information)
        Return True
    End Function
End Class