Imports System.Collections.Generic

Public Class Form1
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim txt_originals As String = ""
        Dim txt_clones As String = ""

        ' Make and clone a Customer.
        Dim cust1 As New Customer("Rod", "Stephens", "1337 Leet St", "Bugsville", "AZ", "87654")
        Dim cust2 As Customer = cust1.Clone()
        txt_originals &= "*** cust1:" & vbCrLf & cust1.ToString() & "----------" & vbCrLf
        txt_clones &= "*** cust2:" & vbCrLf & cust2.ToString() & "----------" & vbCrLf

        ' Make and clone an Order.
        Dim order1 As New Order
        With order1
            .OrderCustomer = New Customer("Bob", "Hacker", "123 Decompile Ct", "Attatash", "ME", "01234")
            .OrderItems.Add(New OrderItem("Cookies", 12))
            .OrderItems.Add(New OrderItem("Pencil", 2))
            .OrderItems.Add(New OrderItem("Notebook", 6))
        End With

        ' Make a shallow clone.
        Dim order2 As Order = order1.Clone(False)

        ' Make a deep clone.
        Dim order3 As Order = order1.Clone(True)

        ' Modify the original Order object.
        With order1.OrderCustomer
            .FirstName = "Mindy"
            .LastName = "Modified"
            .Street = "12 Altered Ave"
            .City = "Changed City"
            .State = "AL"
            .Zip = "98765"
        End With
        With order1.OrderItems(0)
            .Item = "Rice (grain)"
            .Quantity = 10000
        End With

        ' Display the three Orders.
        txt_originals &= "*** order1: " & vbCrLf & order1.ToString() & "----------" & vbCrLf
        txt_clones &= "*** order2: " & vbCrLf & order2.ToString() & "----------" & vbCrLf

        txt_originals &= "*** order1: " & vbCrLf & order1.ToString() & "----------" & vbCrLf
        txt_clones &= "*** order3: " & vbCrLf & order3.ToString() & "----------" & vbCrLf

        ' Display the results.
        txtOriginals.Text = txt_originals
        txtClones.Text = txt_clones
        txtOriginals.Select(0, 0)
        txtClones.Select(0, 0)
    End Sub

    Private Sub Form1_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        Const GAP As Integer = 2
        Dim wid As Integer = (Me.ClientSize.Width - GAP) \ 2
        txtOriginals.SetBounds(0, 0, wid, Me.ClientSize.Height)
        txtClones.SetBounds(Me.ClientSize.Width - wid, 0, wid, Me.ClientSize.Height)
    End Sub
End Class

Public Class Customer
    Public FirstName As String
    Public LastName As String
    Public Street As String
    Public City As String
    Public State As String
    Public Zip As String

    ' Initialize the Customer's properties.
    Public Sub New(ByVal new_firstname As String, ByVal new_lastname As String, ByVal new_street As String, ByVal new_city As String, ByVal new_state As String, ByVal new_zip As String)
        FirstName = new_firstname
        LastName = new_lastname
        Street = new_street
        City = new_city
        State = new_state
        Zip = new_zip
    End Sub

    ' Return a text representation of the Customer.
    Public Overrides Function ToString() As String
        Return FirstName & " " & LastName & vbCrLf & Street & vbCrLf & City & " " & State & " " & Zip & vbCrLf
    End Function

    ' Return a shallow clone.
    Public Function Clone() As Customer
        Return DirectCast(Me.MemberwiseClone(), Customer)
    End Function
End Class

Public Class OrderItem
    Public Item As String
    Public Quantity As Integer

    Public Sub New(ByVal new_item As String, ByVal new_quantity As Integer)
        Item = new_item
        Quantity = new_quantity
    End Sub

    Public Overrides Function ToString() As String
        Return Item & " (" & Quantity.ToString() & ")" & vbCrLf
    End Function

    ' Return a shallow clone.
    Public Function Clone() As OrderItem
        Return DirectCast(Me.MemberwiseClone(), OrderItem)
    End Function
End Class

Public Class Order
    Public OrderCustomer As Customer
    Public OrderItems As New List(Of OrderItem)

    Public Overrides Function ToString() As String
        Dim txt As String = OrderCustomer.ToString
        For Each order_item As OrderItem In OrderItems
            txt &= "    " & order_item.ToString()
        Next order_item

        Return txt
    End Function

    ' Return a deep or shallow clone.
    Public Function Clone(ByVal deep As Boolean) As Order
        ' Start with a shallow clone.
        Dim new_order As Order = DirectCast(Me.MemberwiseClone(), Order)

        ' If appropriate, copy deeper data.
        If deep Then
            With new_order
                ' Clone the OrderCustomer.
                .OrderCustomer = Me.OrderCustomer.Clone()

                ' Clone the OrderItems list.
                .OrderItems = New List(Of OrderItem)
                For Each order_item As OrderItem In Me.OrderItems
                    .OrderItems.Add(order_item.Clone())
                Next order_item
            End With
        End If

        Return new_order
    End Function
End Class

Public Class CloneableClass
    Implements ICloneable

    Public Function Clone() As Object Implements System.ICloneable.Clone
        Return Me
    End Function

    Public Function Clone(ByVal deep As Boolean) As CloneableClass
        ' Return a deep clone.
        If deep Then Return DirectCast(Me.Clone(), CloneableClass)

        ' Return a shallow clone.
        Return DirectCast(Me.MemberwiseClone, CloneableClass)
    End Function
End Class