Imports System.Drawing.Drawing2D

Public Class Form1
    ' This object holds the score data.
    Private m_ScoreModel As ScoreModel

    ' Create models, views, and controllers.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Make the model.
        m_ScoreModel = New ScoreModel(picGraph.ClientSize.Height)

        ' Make a ScoreLabelView.
        Dim label_view As New ScoreLabelView(m_ScoreModel, lblValues)

        ' Make the ScoreGraphViewController.
        Dim graph_view_controller As New ScoreGraphViewController(m_ScoreModel, picGraph)

        ' Make ScoreTextViewControllers.
        Dim tvc As ScoreTextViewController
        tvc = New ScoreTextViewController(m_ScoreModel, txtValue0, 0)
        tvc = New ScoreTextViewController(m_ScoreModel, txtValue1, 1)
        tvc = New ScoreTextViewController(m_ScoreModel, txtValue2, 2)
        tvc = New ScoreTextViewController(m_ScoreModel, txtValue3, 3)
        tvc = New ScoreTextViewController(m_ScoreModel, txtValue4, 4)
        tvc = New ScoreTextViewController(m_ScoreModel, txtValue5, 5)
        tvc = New ScoreTextViewController(m_ScoreModel, txtValue6, 6)
        tvc = New ScoreTextViewController(m_ScoreModel, txtValue7, 7)
        tvc = New ScoreTextViewController(m_ScoreModel, txtValue8, 8)
        tvc = New ScoreTextViewController(m_ScoreModel, txtValue9, 9)

        ' Set some data.
        Dim rand As New Random
        For Each ctl As Control In Me.Controls
            If TypeOf ctl Is TextBox Then
                Dim txt As TextBox = DirectCast(ctl, TextBox)
                txt.Text = rand.Next(10, picGraph.ClientSize.Height).ToString()
            End If
        Next ctl
    End Sub
End Class

' Stores score data.
Public Class ScoreModel
    Private m_MaxValue As Integer

    ' We raise this event when data is modified.
    Public Event DataModified(ByVal index As Integer)

    Public Sub New(ByVal max_value As Integer)
        m_MaxValue = max_value
    End Sub

    ' Get or set a value.
    Public Const MaxValue As Integer = 9
    Private m_Values(0 To MaxValue) As Integer
    Public Property Value(ByVal index As Integer) As Integer
        Get
            Return m_Values(index)
        End Get
        Set(ByVal value As Integer)
            If value < 0 Then value = 0
            If value > m_MaxValue Then value = m_MaxValue
            m_Values(index) = value

            ' Notify views.
            RaiseEvent DataModified(index)
        End Set
    End Property
End Class

' Modifies score data.
Public Class ScoreController
    Private m_ScoreModel As ScoreModel

    Public Sub New(ByVal score_model As ScoreModel)
        m_ScoreModel = score_model
    End Sub

    ' Set a score value.
    Public WriteOnly Property Value(ByVal index As Integer) As Integer
        Set(ByVal value As Integer)
            m_ScoreModel.Value(index) = value
        End Set
    End Property
End Class

' Displays score data.
Public MustInherit Class ScoreView
    Protected WithEvents m_ScoreModel As ScoreModel
    Public Sub New(ByVal score_model As ScoreModel)
        m_ScoreModel = score_model
    End Sub

    ' Redisplay the data.
    Protected MustOverride Sub m_ScoreModel_DataModified(ByVal index As Integer) Handles m_ScoreModel.DataModified
End Class

' Displays score data in a label.
Public Class ScoreLabelView
    Inherits ScoreView

    Private m_Label As Label

    ' Save the ScoreModel and Label.
    Public Sub New(ByVal score_model As ScoreModel, ByVal the_label As Label)
        MyBase.new(score_model)
        m_Label = the_label
    End Sub

    ' Display the data in the label.
    Protected Overrides Sub m_ScoreModel_DataModified(ByVal index As Integer)
        Dim txt As String = ""
        For i As Integer = 0 To ScoreModel.MaxValue
            txt &= m_ScoreModel.Value(i).ToString & " "
        Next i
        m_Label.Text = txt
    End Sub
End Class

' Displays score data in a graph.
Public Class ScoreGraphView
    Inherits ScoreView

    ' The PictureBox we draw on.
    Private WithEvents m_Canvas As PictureBox

    ' Save the ScoreModel and Label.
    Public Sub New(ByVal score_model As ScoreModel, ByVal picture_box As PictureBox)
        MyBase.new(score_model)
        m_Canvas = picture_box
    End Sub

    ' Display the data in a graph.
    Protected Overrides Sub m_ScoreModel_DataModified(ByVal index As Integer)
        m_Canvas.Invalidate()
    End Sub

    ' Draw the graph.
    Private Sub m_Canvas_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles m_Canvas.Paint
        e.Graphics.Clear(m_Canvas.BackColor)
        Dim dx As Integer = m_Canvas.ClientSize.Width \ (ScoreModel.MaxValue + 1)
        Dim hgt As Integer = m_Canvas.ClientSize.Height
        For i As Integer = 0 To ScoreModel.MaxValue
            Using br As New HatchBrush(CType(i, HatchStyle), Color.Black, Color.White)
                e.Graphics.FillRectangle(br, i * dx, hgt - m_ScoreModel.Value(i) - 1, dx, hgt)
            End Using
            e.Graphics.DrawRectangle(Pens.Black, i * dx, hgt - m_ScoreModel.Value(i) - 1, dx, hgt)
        Next i
    End Sub
End Class

' Acts as a ScoreGraphView and a ScoreController.
Public Class ScoreGraphViewController
    Private m_ScoreGraphView As ScoreGraphView
    Private m_ScoreController As ScoreController

    ' We need to catch MouseDown events on the graph.
    Private WithEvents m_Canvas As PictureBox

    ' Save the ScoreModel and Label.
    Public Sub New(ByVal score_model As ScoreModel, ByVal picture_box As PictureBox)
        m_Canvas = picture_box
        m_ScoreGraphView = New ScoreGraphView(score_model, picture_box)
        m_ScoreController = New ScoreController(score_model)
    End Sub

    ' The index we are changing.
    Private m_Index As Integer = -1

    ' Change the clicked value.
    Private Sub SetValue(ByVal y As Integer)
        If y < 1 Then y = 1
        If y > m_Canvas.ClientSize.Height Then y = m_Canvas.ClientSize.Height
        m_ScoreController.Value(m_Index) = m_Canvas.ClientSize.Height - y
    End Sub
    Private Sub m_Canvas_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles m_Canvas.MouseDown
        Dim dx As Integer = m_Canvas.ClientSize.Width \ (ScoreModel.MaxValue + 1)
        m_Index = e.X \ dx
        If m_Index > ScoreModel.MaxValue Then m_Index = ScoreModel.MaxValue

        SetValue(e.Y)
    End Sub
    Private Sub m_Canvas_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles m_Canvas.MouseMove
        If m_Index < 0 Then Exit Sub
        SetValue(e.Y)
    End Sub
    Private Sub m_Canvas_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles m_Canvas.MouseUp
        m_Index = -1
    End Sub
End Class

' Displays score data in a label.
Public Class ScoreTextBoxView
    Inherits ScoreView

    ' The TextBox where we display data.
    Private m_TextBox As TextBox

    ' The index of the value we display.
    Private m_Index As Integer

    ' Save the ScoreModel and TextBox.
    Public Sub New(ByVal score_model As ScoreModel, ByVal text_box As TextBox, ByVal index As Integer)
        MyBase.new(score_model)
        m_TextBox = text_box
        m_Index = index
    End Sub

    ' Display the data in the TextBox.
    Protected Overrides Sub m_ScoreModel_DataModified(ByVal index As Integer)
        If m_Index <> index Then Exit Sub
        m_TextBox.Text = m_ScoreModel.Value(index).ToString
    End Sub
End Class

' Acts as a ScoreTextView and a ScoreController.
Public Class ScoreTextViewController
    Private m_ScoreTextBoxView As ScoreTextBoxView
    Private m_ScoreController As ScoreController

    ' We need to catch TextChanged events in the TextBox.
    Private WithEvents m_TextBox As TextBox

    ' The index of the value in the model that we control.
    Private m_Index As Integer

    ' Save the ScoreModel and Label.
    Public Sub New(ByVal score_model As ScoreModel, ByVal text_box As TextBox, ByVal index As Integer)
        m_TextBox = text_box
        m_Index = index
        m_ScoreTextBoxView = New ScoreTextBoxView(score_model, text_box, index)
        m_ScoreController = New ScoreController(score_model)
    End Sub

    ' Change the value.
    Private Sub m_TextBox_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles m_TextBox.TextChanged
        If IsNumeric(m_TextBox.Text) Then
            m_ScoreController.Value(m_Index) = CInt(m_TextBox.Text)
        End If
    End Sub
End Class
