Imports System.CodeDom.Compiler
Imports System.Text
Imports System.Reflection

Public Class SimplerCodeRunner
    ' Errors generated by the last run.
    Public CompilerErrors As New CompilerErrorCollection

    ' Array of references needed by the code.
    Public References As New Collection

    ' Object to be passed to the Main function.
    Public MainParameter As Object

    Public Function Run(ByVal vb_code As String) As Object
        ' Prepare the compiler parameters.
        ' Add any needed references.
        Dim compiler_params As CompilerParameters = New CompilerParameters
        For Each reference As String In References
            compiler_params.ReferencedAssemblies.Add(reference)
        Next reference

        ' Generate a library in memory.
        compiler_params.CompilerOptions = "/t:library"
        compiler_params.GenerateInMemory = True

        ' Indent the entered code to make it easier to read.
        vb_code = "      " & vb_code.Replace(vbCrLf, vbCrLf & "      ").Trim()

        ' Insert the code inside a class and function.
        vb_code = _
            "Imports System" & vbCrLf & _
            "Imports System.Math" & vbCrLf & _
            "Imports System.Windows.Forms" & vbCrLf & vbCrLf & _
            "Namespace MainNamespace" & vbCrLf & _
            "  Class MainClass" & vbCrLf & _
            "    Public Function Main(ByVal the_picture As Object) As Object" & vbCrLf & _
            vb_code & vbCrLf & _
            "    End Function" & vbCrLf & _
            "  End Class" & vbCrLf & _
            "End Namespace"

        ' Compile the code.
        Debug.WriteLine(vb_code)
        Dim code_provider As VBCodeProvider = New VBCodeProvider
        Dim compiler_results As CompilerResults = _
            code_provider.CompileAssemblyFromSource(compiler_params, vb_code)
        code_provider.Dispose()

        ' Check for errors.
        CompilerErrors = compiler_results.Errors
        If compiler_results.Errors.Count > 0 Then
            ' There were errors.
            Throw New CompilerException("Compile Errors")
        Else
            ' There were no errors. Try to execute the code.
            ' Get the assembly.
            Dim the_assembly As System.Reflection.Assembly = _
                compiler_results.CompiledAssembly

            ' Make a MainClass object.
            Dim main_object As Object = _
                the_assembly.CreateInstance("MainNamespace.MainClass")

            ' Get the MainClass's type.
            Dim main_object_type As Type = main_object.GetType

            ' Get the MainClass's Main method.
            Dim run_method_info As MethodInfo
            run_method_info = main_object_type.GetMethod("Main")

            ' Execute the method and return the result.
            Dim params() As Object = Nothing
            If MainParameter IsNot Nothing Then
                params = New Object() {MainParameter}
            End If
            Return run_method_info.Invoke(main_object, params)
        End If
    End Function
End Class

Public Class CompilerException
    Inherits ApplicationException
    Public Sub New()
        MyBase.New("Compiler Exception")
    End Sub
    Public Sub New(ByVal message As String)
        MyBase.New(message)
    End Sub
End Class