Public Class AHlsColorDialogForm
    Private m_IgnoreEvents As Boolean = False

    ' Get or set the dialog's Color value.
    Public Property Color() As Color
        Get
            ' Convert the HLS color to RGB.
            Dim r, g, b As Double
            ColorConversions.HlsToRgb( _
                HsColorWheel1.Hue, _
                LightnessBar1.Lightness, _
                HsColorWheel1.Saturation, _
                r, g, b)
            Return System.Drawing.Color.FromArgb(CInt(AlphaBar1.Alpha), CInt(r), CInt(g), CInt(b))
        End Get
        Set(ByVal Value As Color)
            SetColorARGB(Value.A, Value.R, Value.G, Value.B)
        End Set
    End Property

    Private Sub SetColorARGB(ByVal a As Double, ByVal r As Double, ByVal g As Double, ByVal b As Double)
        ' Convert the RGB to HLS.
        Dim h, l, s As Double
        ColorConversions.RgbToHls( _
            r, g, b, h, l, s)

        ' Set the color.
        SetColorAHLS(a, h, l, s)
    End Sub

    Private Sub SetColorAHLS(ByVal a As Double, ByVal h As Double, ByVal l As Double, ByVal s As Double)
        ' Initialize the controls.
        HsColorWheel1.Hue = CSng(h)
        HsColorWheel1.Saturation = CSng(s)

        ' Update the lightness bar.
        LightnessBar1.Lightness = CSng(l)
        SetLightnessBaseColor()

        ' Update the alpha bar.
        AlphaBar1.Alpha = CSng(a)

        ' Update the scrollbars.
        UpdateScrollBars()
    End Sub

    ' Update the lightness bar, alpha bar, and sample.
    Private Sub SetLightnessBaseColor()
        ' Convert to RGB with lightness = 0.5.
        Dim r, g, b As Double
        ColorConversions.HlsToRgb( _
            HsColorWheel1.Hue, _
            0.5, _
            HsColorWheel1.Saturation, _
            r, g, b)

        ' Set the lightness bar's base color.
        LightnessBar1.BaseColor = System.Drawing.Color.FromArgb(255, CInt(r), CInt(g), CInt(b))

        ' Set the alpha bar's base color.
        AlphaBar1.BaseColor = System.Drawing.Color.FromArgb(255, CInt(r), CInt(g), CInt(b))

        ' Display a sample.
        picSample.Invalidate()
    End Sub

    ' Set the scroll bars for the current HLS value.
    Private Sub UpdateScrollBars()
        If m_IgnoreEvents Then Exit Sub
        m_IgnoreEvents = True

        hbarHue.Value = CInt(Int(HsColorWheel1.Hue))
        hbarLightness.Value = CInt(Int(359 * LightnessBar1.Lightness))
        hbarSaturation.Value = CInt(Int(359 * HsColorWheel1.Saturation))

        ' Convert to RGB.
        Dim r, g, b As Double
        ColorConversions.HlsToRgb( _
            HsColorWheel1.Hue, _
            LightnessBar1.Lightness, _
            HsColorWheel1.Saturation, _
            r, g, b)
        hbarRed.Value = CInt(r)
        hbarGreen.Value = CInt(g)
        hbarBlue.Value = CInt(b)

        m_IgnoreEvents = False
    End Sub

    ' The hue/saturation has changed. 
    Private Sub HsColorWheel1_ColorScrolled(ByVal hue As System.Single, ByVal saturation As System.Single) Handles HsColorWheel1.ColorScrolled
        SetColorAHLS( _
            AlphaBar1.Alpha, _
            HsColorWheel1.Hue, _
            LightnessBar1.Lightness, _
            HsColorWheel1.Saturation)
    End Sub

    ' The lightness has changed. Update the sample.
    Private Sub LightnessBar1_LightnessScrolled(ByVal new_lightness As System.Single) Handles LightnessBar1.LightnessScrolled
        SetColorAHLS( _
            AlphaBar1.Alpha, _
            HsColorWheel1.Hue, _
            LightnessBar1.Lightness, _
            HsColorWheel1.Saturation)
    End Sub

    ' Handle HLS scroll bar events.
    Private Sub hbarHue_Scroll(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles hbarHue.Scroll
        If m_IgnoreEvents Then Exit Sub
        SetColorAHLS( _
            AlphaBar1.Alpha, _
            hbarHue.Value, _
            LightnessBar1.Lightness, _
            HsColorWheel1.Saturation)
    End Sub
    Private Sub hbarSaturation_Scroll(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles hbarSaturation.Scroll
        If m_IgnoreEvents Then Exit Sub
        SetColorAHLS( _
            AlphaBar1.Alpha, _
            HsColorWheel1.Hue, _
            LightnessBar1.Lightness, _
            hbarSaturation.Value / 359)
    End Sub
    Private Sub hbarLightness_Scroll(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles hbarLightness.Scroll
        If m_IgnoreEvents Then Exit Sub
        SetColorAHLS( _
            AlphaBar1.Alpha, _
            HsColorWheel1.Hue, _
            hbarLightness.Value / 359, _
            HsColorWheel1.Saturation)
    End Sub

    ' Handle HLS scroll bar events.
    Private Sub hbarRGB_Scroll(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles hbarRed.Scroll, hbarGreen.Scroll, hbarBlue.Scroll
        If m_IgnoreEvents Then Exit Sub
        SetColorARGB(AlphaBar1.Alpha, _
            hbarRed.Value, hbarGreen.Value, hbarBlue.Value)
    End Sub

    ' Handle a change in alpha value.
    Private Sub AlphaBar1_AlphaScrolled(ByVal new_alpha As System.Single) Handles AlphaBar1.AlphaScrolled
        SetColorAHLS( _
            new_alpha, _
            HsColorWheel1.Hue, _
            LightnessBar1.Lightness, _
            HsColorWheel1.Saturation)
    End Sub

    ' Display the sample.
    Private Sub picSample_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles picSample.Paint
        ' Draw a tiled background.
        Dim bg_brush As New TextureBrush(AlphaBar1.TileImage)
        e.Graphics.FillRectangle(bg_brush, picSample.ClientRectangle)
        bg_brush.Dispose()

        ' Cover with the sample color.
        Dim clr As Color = LightnessBar1.SelectedColor
        clr = System.Drawing.Color.FromArgb(CInt(AlphaBar1.Alpha), clr.R, clr.G, clr.B)
        Dim fg_brush As New SolidBrush(clr)
        e.Graphics.FillRectangle(fg_brush, picSample.ClientRectangle)
        fg_brush.Dispose()
    End Sub
End Class