Imports System.Math
Imports System.Drawing.Drawing2D
Imports System.ComponentModel
Imports System.Runtime.InteropServices

<ToolboxBitmap(GetType(AlphaBar), "tbxAlphaBar")> _
<DefaultEvent("AlphaScrolled")> _
Public Class AlphaBar
    Inherits System.Windows.Forms.Control

    ' Raised when the user moves the mouse with the left button down.
    Public Event AlphaScrolled(ByVal new_alpha As Single)

    ' Raised when the user releases the mouse.
    Public Event AlphaPicked(ByVal new_alpha As Single)

    ' The width of the triangle that shows the selected Alpha.
    Private Const TRI_WID As Integer = 5
    Private Const TRI_MARGIN As Integer = 1

    ' m_Alpha is between 0 (transparent) and 1 (opaque).
    ' Alpha is between 0 and 255.
    Private m_Alpha As Single = 0.5
    Public Property Alpha() As Single
        Get
            Return m_Alpha * 255
        End Get
        Set(ByVal Value As Single)
            m_Alpha = Value / 255
            If m_Alpha < 0 Then m_Alpha = 0
            If m_Alpha > 1 Then m_Alpha = 1
            Me.Invalidate() ' Redraw.
        End Set
    End Property

    ' The basic color.
    Private m_BaseColor As Color = Color.Blue
    Public Property BaseColor() As Color
        Get
            Return m_BaseColor
        End Get
        Set(ByVal Value As Color)
            m_BaseColor = Value
            Me.Invalidate() ' Redraw.
        End Set
    End Property

    ' The background brush image.
    Private m_TileImage As Image = Nothing
    <DefaultValue(GetType(Image), Nothing)> _
    Public Property TileImage() As Image
        Get
            If Not (m_TileImage Is Nothing) Then Return m_TileImage

            ' Make a default image if necessary.
            Dim bm As New Bitmap(12, 12)
            Dim gr As Graphics = Graphics.FromImage(bm)
            gr.FillRectangle(Brushes.Black, 0, 0, 6, 6)
            gr.FillRectangle(Brushes.Cyan, 6, 0, 6, 6)
            gr.FillRectangle(Brushes.Orange, 0, 6, 6, 6)
            gr.FillRectangle(Brushes.Black, 6, 6, 6, 6)
            gr.Dispose()
            Return bm
        End Get
        Set(ByVal Value As Image)
            m_TileImage = Value
            Me.Invalidate() ' Redraw.
        End Set
    End Property

    ' Force a redraw.
    Private Sub AlphaBar_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        ' Use double-buffering to prevent flicker.
        Me.SetStyle( _
            ControlStyles.AllPaintingInWmPaint Or _
            ControlStyles.UserPaint Or _
            ControlStyles.DoubleBuffer, _
            True)
        Me.UpdateStyles()

        ' Redraw.
        Me.Invalidate()
    End Sub

    ' Redraw.
    Private Sub AlphaBar_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles MyBase.Paint
        e.Graphics.Clear(Me.BackColor)

        ' Fill with the tile image.
        Dim tile_image As Image = TileImage
        Dim rect As New RectangleF(0, 0, tile_image.Width, Me.Height)
        Dim bg_br As New TextureBrush(tile_image)
        e.Graphics.FillRectangle(bg_br, rect)
        bg_br.Dispose()

        ' Fill with the selected color at different alpha values.
        Dim color1 As Color = Color.FromArgb(255, _
            m_BaseColor.R, m_BaseColor.G, m_BaseColor.B)
        Dim color2 As Color = Color.FromArgb(0, _
            m_BaseColor.R, m_BaseColor.G, m_BaseColor.B)
        Dim fg_brush As New LinearGradientBrush(rect, _
            color1, color2, _
            LinearGradientMode.Vertical)
        e.Graphics.FillRectangle(fg_brush, rect)
        fg_brush.Dispose()

        ' Draw the triangle.
        Dim x As Integer = tile_image.Width + TRI_MARGIN
        Dim y As Integer = CInt(Me.Height * (1 - m_Alpha))
        Dim pts() As Point = { _
            New Point(x, y), _
            New Point(x + TRI_WID, y - TRI_WID), _
            New Point(x + TRI_WID, y + TRI_WID), _
            New Point(x, y) _
        }
        Dim fore_brush As New SolidBrush(Me.ForeColor)
        e.Graphics.FillPolygon(fore_brush, pts)
        fore_brush.Dispose()
    End Sub

    Private Sub AlphaBar_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseMove, MyBase.MouseDown
        If e.Button = Windows.Forms.MouseButtons.Left Then
            SavePoint(e.Y)
        End If
    End Sub

    ' Save this point.
    Private Sub SavePoint(ByVal y As Integer)
        ' Get the corresponding Alpha value.
        m_Alpha = CSng((Me.Height - y) / Me.Height)
        If m_Alpha < 0 Then m_Alpha = 0
        If m_Alpha > 1 Then m_Alpha = 1

        ' Redraw.
        Me.Invalidate()

        ' Raise the AlphaScrolled event.
        RaiseEvent AlphaScrolled(Alpha)
    End Sub

    ' Raise the AlphaPicked event.
    Private Sub HsColorWheel_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseUp
        RaiseEvent AlphaPicked(Alpha)
    End Sub
End Class
