Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Text.RegularExpressions
Imports System.Drawing
Imports System.Windows.Forms

<ToolboxBitmap(GetType(RegexValidationProvider), "tbxRegexValidationProvider")> _
<ProvideProperty("RegexPattern", GetType(TextBox))> _
<ProvideProperty("RegexMessage", GetType(TextBox))> _
<ProvideProperty("RegexAllowBlank", GetType(TextBox))> _
Public Class RegexValidationProvider
    Implements IExtenderProvider

    ' Store a control's regular expression pattern and error message.
    Private Const ALLOW_BLANK_DEFAULT As Boolean = True
    Private Class RegexInfo
        Public Pattern As String = ""
        Public Message As String = ""
        Public AllowBlank As Boolean = ALLOW_BLANK_DEFAULT
        Public Function IsBlank() As Boolean
            Return (Pattern.Length = 0) AndAlso _
                   (Message.Length = 0) AndAlso _
                   (AllowBlank = ALLOW_BLANK_DEFAULT)
        End Function
    End Class

    ' Hold information about controls and their regular expressions.
    Private m_RegexInfos As New Generic.Dictionary(Of TextBox, RegexInfo)

    ' Extend only TextBoxes.
    Public Function CanExtend(ByVal extendee As Object) As Boolean Implements System.ComponentModel.IExtenderProvider.CanExtend
        Return (TypeOf extendee Is TextBox)
    End Function

#Region "RegexPattern Property"
    ' Return a TextBox's RegexPattern value.
    Public Function GetRegexPattern(ByVal client_control As TextBox) As String
        If m_RegexInfos.ContainsKey(client_control) Then
            Return m_RegexInfos.Item(client_control).Pattern
        Else
            Return ""
        End If
    End Function

    ' Set a TextBox's RegexPattern value.
    Public Sub SetRegexPattern(ByVal client_control As TextBox, ByVal new_pattern As String)
        If new_pattern Is Nothing Then new_pattern = ""

        ' Get the client's RegexInfo object or make a new one.
        Dim regex_info As RegexInfo = GetOrMakeRegexInfo(client_control)

        ' Save the new pattern.
        regex_info.Pattern = new_pattern

        ' Add or remove the entry if necessary.
        AddOrRemoveIfNecessary(client_control, regex_info)
    End Sub
#End Region ' RegexPattern Property

#Region "RegexMessage Property"
    ' Return a TextBox's RegexMessage value.
    Public Function GetRegexMessage(ByVal client_control As TextBox) As String
        If m_RegexInfos.ContainsKey(client_control) Then
            Return m_RegexInfos.Item(client_control).Message
        Else
            Return ""
        End If
    End Function

    ' Set a TextBox's RegexMessage value.
    Public Sub SetRegexMessage(ByVal client_control As TextBox, ByVal new_message As String)
        If new_message Is Nothing Then new_message = ""

        ' Get the client's RegexInfo object or make a new one.
        Dim regex_info As RegexInfo = GetOrMakeRegexInfo(client_control)

        ' Save the new message.
        regex_info.Message = new_message

        ' Add or remove the entry if necessary.
        AddOrRemoveIfNecessary(client_control, regex_info)
    End Sub
#End Region ' RegexMessage Property

#Region "RegexAllowBlank Property"
    ' Return a TextBox's RegexAllowBlank value.
    Public Function GetRegexAllowBlank(ByVal client_control As TextBox) As Boolean
        If m_RegexInfos.ContainsKey(client_control) Then
            Return m_RegexInfos.Item(client_control).AllowBlank
        Else
            Return ALLOW_BLANK_DEFAULT
        End If
    End Function

    ' Set a TextBox's RegexAllowBlank value.
    Public Sub SetRegexAllowBlank(ByVal client_control As TextBox, ByVal new_AllowBlank As Boolean)
        ' Get the client's RegexInfo object or make a new one.
        Dim regex_info As RegexInfo = GetOrMakeRegexInfo(client_control)

        ' Save the new AllowBlank.
        regex_info.AllowBlank = new_AllowBlank

        ' Add or remove the entry if necessary.
        AddOrRemoveIfNecessary(client_control, regex_info)
    End Sub
#End Region ' RegexAllowBlank Property

    ' Return the client's RegexInfo or make a new one.
    Private Function GetOrMakeRegexInfo(ByVal client_control As TextBox) As RegexInfo
        If m_RegexInfos.ContainsKey(client_control) Then
            Return m_RegexInfos.Item(client_control)
        Else
            Return New RegexInfo
        End If
    End Function

    ' Add or remove the entry if necessary.
    Private Sub AddOrRemoveIfNecessary(ByVal client_control As TextBox, ByVal regex_info As RegexInfo)
        ' See if it's all blank.
        If regex_info.IsBlank() Then
            ' Remove the TextBox from the expression dictionary.
            If m_RegexInfos.ContainsKey(client_control) Then
                m_RegexInfos.Remove(client_control)
                RemoveHandler client_control.Validating, AddressOf Client_Validating
            End If
        Else
            ' Add or update the control's expression.
            m_RegexInfos.Item(client_control) = regex_info
            AddHandler client_control.Validating, AddressOf Client_Validating
        End If
    End Sub

    ' A client is being validated. See if it matches its expression.
    Private Sub Client_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs)
        ' Get the TextBox and its information.
        Dim text_box As TextBox = DirectCast(sender, TextBox)
        If Not m_RegexInfos.ContainsKey(text_box) Then Exit Sub
        Dim regex_info As RegexInfo = m_RegexInfos(text_box)

        ' Validate the expression.
        If regex_info.AllowBlank And text_box.Text.Length = 0 Then
            ' Blank is okay. Clear any error.
            errRegex.SetError(text_box, "")
        ElseIf Regex.IsMatch(text_box.Text, regex_info.Pattern, RegexOptions.IgnoreCase) Then
            ' It matches. Clear any error.
            errRegex.SetError(text_box, "")
        Else
            ' It doesn't match. Display the error message.
            If regex_info.Message.Length = 0 Then
                errRegex.SetError(text_box, "Invalid format")
            Else
                errRegex.SetError(text_box, regex_info.Message)
            End If
        End If
    End Sub

End Class
