''' <summary>
''' Represents a person.
'''   <seealso cref="Customer">Derived Customer class</seealso>
''' </summary>
''' <remarks>
''' This is an abstract class.
'''   <seealso cref="Employee">Derived Employee class</seealso>
''' </remarks>
''' <scope>Public</scope>
''' <History>
'''   <Change By="Rod Stephens" Date="4/1/2007">Initial creation.</Change>
''' </History>
Public MustInherit Class Person
    ''' <summary>
    ''' Initialize a new Person.
    ''' </summary>
    ''' <param name="new_FirstName">The new object's <see cref="FirstName">FirstName</see> property.</param>
    ''' <param name="new_LastName">The new object's <see cref="LastName">LastName</see> property.</param>
    ''' <remarks>
    ''' Use this constructor to create and initialize the object in one step.
    ''' <para/>
    ''' Here's the verbose way:
    '''   <code>
    '''     Dim person1 As New Person()
    '''     person1.FirstName = "Eileen"
    '''     person1.LastName = "Leet"
    '''   </code>
    ''' Here's the concise method:
    '''   <code>
    '''     Dim person1 As New Person("Eileen", "Leet")
    '''   </code>
    ''' </remarks>
    ''' <scope>Public</scope>
    Public Sub New(ByVal new_FirstName As String, ByVal new_LastName As String)
        FirstName = new_FirstName
        LastName = new_LastName
    End Sub

    ''' <summary>
    ''' The default constructor.
    ''' </summary>
    ''' <remarks>This constructor does nothing.</remarks>
    ''' <scope>Public</scope>
    Public Sub New()

    End Sub

    ''' <summary>
    ''' The private variable that holds the FirstName value.
    ''' </summary>
    ''' <remarks>Use the property procedures to get and set this value.</remarks>
    ''' <scope>Private</scope>
    Private m_FirstName As String
    ''' <summary>
    ''' The Person's first name.
    ''' </summary>
    ''' <value>The Person's first or given name.</value>
    ''' <returns>A person's first name.</returns>
    ''' <remarks>Use this to set the <c>Person</c>'s first name. Use as in:
    '''   <code>
    '''     a_person.FirstName = "Arthur"
    '''     a_person.LastName = "Abend"
    '''   </code>
    ''' </remarks>
    ''' <scope>Public</scope>
    Public Property FirstName() As String
        Get
            Return m_FirstName
        End Get
        Set(ByVal value As String)
            m_FirstName = value
        End Set
    End Property

    ''' <summary>
    ''' The private variable that holds the LastName value.
    ''' </summary>
    ''' <remarks>Use the property procedures to get and set this value.</remarks>
    ''' <scope>Private</scope>
    Private m_LastName As String
    ''' <summary>
    ''' The Person's last name.
    ''' </summary>
    ''' <value>The Person's last or family name.</value>
    ''' <returns>A person's last name.</returns>
    ''' <remarks>Use this to set the Person's last name.</remarks>
    ''' <scope>Public</scope>
    Public Property LastName() As String
        Get
            Return m_LastName
        End Get
        Set(ByVal value As String)
            m_LastName = value
        End Set
    End Property
End Class
