Imports System.Drawing.Printing

Module PrintFormTools
    Private Structure RECT
        Public Left As Integer
        Public Top As Integer
        Public Right As Integer
        Public Bottom As Integer
    End Structure

    Private Declare Function GetWindowRect Lib "user32" (ByVal hWnd As Integer, ByRef lpRect As RECT) As Integer
    Private Declare Function BitBlt Lib "gdi32" Alias "BitBlt" (ByVal hDestDC As Integer, ByVal x As Integer, ByVal y As Integer, ByVal nWidth As Integer, ByVal nHeight As Integer, ByVal hSrcDC As Integer, ByVal xSrc As Integer, ByVal ySrc As Integer, ByVal dwRop As Integer) As Integer
    Private Declare Function GetWindowDC Lib "user32" Alias "GetWindowDC" (ByVal hwnd As Integer) As Integer
    Private Declare Function ReleaseDC Lib "user32" (ByVal hwnd As Integer, ByVal hdc As Integer) As Integer

    ' The form's image.
    Private m_Bitmap As Bitmap
    Private m_EnlargeToFit As Boolean

    ' The PrintDocument object that does the printing.
    Private WithEvents m_PrintDocument As PrintDocument

    ' Display a print preview of the form.
    Public Sub PrintPreviewForm(ByVal dlg As PrintPreviewDialog, ByVal frm As Form, Optional ByVal enlarge_to_fit As Boolean = False)
        ' Save the form's image now so the PrintPreview
        ' dialog doesn't obscure part of it.
        m_Bitmap = GetWindowImage(frm.Handle.ToInt32)
        m_EnlargeToFit = enlarge_to_fit

        ' Make the PrintDocument object to print.
        m_PrintDocument = New PrintDocument
        dlg.Document = m_PrintDocument
        dlg.ShowDialog()
    End Sub

    ' Return a Bitmap containing the window's image.
    Public Function GetWindowImage(ByVal hWnd As Integer) As Bitmap
        ' Get the window's rectangle.
        Dim r As New RECT
        GetWindowRect(hWnd, r)
        Dim wid As Integer = r.Right - r.Left
        Dim hgt As Integer = r.Bottom - r.Top

        ' Make a Bitmap of the right size and 
        ' an associated Graphics object.
        Dim bm As New Bitmap(wid, hgt)
        Dim gr As Graphics = Graphics.FromImage(bm)

        ' Copy the image from the window to the Bitmap.
        Dim src_hdc As Integer = GetWindowDC(hWnd)
        Dim bm_hdc As IntPtr = gr.GetHdc
        Const SRCCOPY As Integer = &HCC0020
        BitBlt(bm_hdc.ToInt32, 0, 0, wid, hgt, _
            src_hdc, 0, 0, SRCCOPY)

        ' Clean up.
        gr.ReleaseHdc(bm_hdc)
        ReleaseDC(hWnd, src_hdc)

        ' Return the bitmap.
        Return bm
    End Function

    ' Print the form's image.
    Private Sub m_PrintDocument_PrintPage(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs) Handles m_PrintDocument.PrintPage
        ' Get the image's size in 100ths of inches
        ' (the printer's units).
        Dim wid As Single = m_Bitmap.Width / m_Bitmap.HorizontalResolution * 100
        Dim hgt As Single = m_Bitmap.Height / m_Bitmap.VerticalResolution * 100

        ' Draw the image on the page.
        If Not m_EnlargeToFit Then
            ' Center the image.
            Dim x As Single = (e.MarginBounds.Width - wid) / 2 + e.MarginBounds.Left
            Dim y As Single = (e.MarginBounds.Height - hgt) / 2 + e.MarginBounds.Top
            e.Graphics().DrawImage(m_Bitmap, x, y)
        Else
            ' Enlarge the image as much as possible
            ' but preserve the aspect ratio.
            Dim dest_rect As RectangleF
            If wid / hgt > e.MarginBounds.Width / e.MarginBounds.Height Then
                ' The image is relatively short and wide.
                ' Make it as wide as possible.
                With dest_rect
                    .Width = e.MarginBounds.Width
                    .Height = hgt * .Width / wid
                    .X = e.MarginBounds.Left
                    .Y = e.MarginBounds.Top + (e.MarginBounds.Height - .Height) / 2
                End With
            Else
                ' The image is relatively tall and thin.
                ' Make it as tall as possible.
                With dest_rect
                    .Height = e.MarginBounds.Height
                    .Width = wid * .Height / hgt
                    .X = e.MarginBounds.Left + (e.MarginBounds.Width - .Width) / 2
                    .Y = e.MarginBounds.Top
                End With
            End If

            ' Draw the image.
            e.Graphics().DrawImage(m_Bitmap, dest_rect)
        End If

        ' Debugging: Draw the margins.
        'e.Graphics().DrawRectangle(Pens.Red, _
        '    e.MarginBounds.Left, e.MarginBounds.Top, _
        '    e.MarginBounds.Width, e.MarginBounds.Height)

        ' Discard the image.
        m_Bitmap = Nothing

        ' We don't need any more PrintDocument events.
        m_PrintDocument = Nothing
    End Sub
End Module
