Imports System.Collections.Generic

Public Class Form1
    ' Display a print preview.
    Private Sub btnPreview_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPreview.Click
        ' Set the print preview control's PrintDocument object.
        pviewWrapped.Document = pdocWrapped

        ' Display the preview.
        ' The PrintDocument object does the real work.
        pviewWrapped.ShowDialog()
    End Sub

    ' Print.
    Private Sub btnPrint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPrint.Click
        ' Use the PrintDocument to print.
        pdocWrapped.Print()
    End Sub

#Region "Printing Code"
    ' The lines we will print.
    Private m_Paragraphs As List(Of ParagraphInfo)

    ' The number of the next page we will print.
    Private m_NextPage As Integer

    ' The distance of the page number from the
    ' top and left edges of the margin area.
    Private Const PAGE_NUM_MARGIN_X As Integer = 50
    Private Const PAGE_NUM_MARGIN_Y As Integer = 50

    ' Fonts we use while printing.
    Private m_Fonts(0 To 2) As Font

    ' Prepare to print.
    Private Sub pdlgWrap_BeginPrint(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintEventArgs) Handles pdocWrapped.BeginPrint
        ' Create the text to print.
        m_Paragraphs = New List(Of ParagraphInfo)
        m_Paragraphs.Add(New ParagraphInfo(0, 20, 20, _
            "19. Splash Screens"))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "The first thing a user sees of an application is its splash screen. As the adage says, ""You only get one change to make a first impression,"" so it's important that your splash screen represent your application appropriately. While the user only gets one first impression of your application, the splash screen appears every time the application starts so its important that it has a positive impact each time the user sees it."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "A typical splash screen displays some basic information about the application such as its name, version number, and copyright information. Splash screens often provide contact information so users can get technical support if necessary."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "In addition to giving the user a little information, a splash screen gives the user something to look at while the application loads. If a program takes several seconds to connect to databases, look for new information on Web Services, or parse XML data, then a splash screen shows the user that something is happening. If it includes a progress bar or status animation, it can let the user know that the program is working and has not become stuck."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "A well-crafted splash screen shows the users that you pay attention to details and makes an application look more professional. I usually build a splash screen for a new application as soon as development starts so customers viewing prototypes can see it. In one application for the State of Minnesota, I made a splash screen in the shape of the states map. After the initial demonstration, the first question they asked was, ""How did you make that cool splash screen?"""))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "(The second question they asked was, ""How long did it take?"" They liked the splash screen but didnt want to waste money on frills. As youll see in this chapter, making a shaped splash screen only takes a few minutes.)"))
        'm_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
        '    "This chapter explains how to build and use splash screens in Visual Basic. It tells how to add interesting and unusual features to a splash screen such as a shaped form, rotated text, and text filled with a color gradient or pattern. Adding all of these features to every form in a project would probably make the application appear cluttered and distracting but in a splash screen they can make an otherwise utilitarian form more interesting."))
        m_Paragraphs.Add(New ParagraphInfo(0, 20, 20, _
            "Determining Splash Screen Contents"))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "Figure 19-1 shows the splash screen displayed by the ShapedSplashScreen example application. The form is shaped to fit an image on the left and has rounded corners. It displays the applications title eJack in outlined text that is filled with a color gradient shading from red to yellow to green (it looks much better on the screen than in this book). It displays the programs version, serial number, and copyright information. It also displays a progress bar to show the user how far the application has gotten in loading its data."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "A programs About dialog should also display the same information. It should display the programs name, version, serial number, and copyright statement. Because the About dialog contains the same information as the splash screen, it makes sense to use the same form for both."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "Figure 19-2 shows the ShapedSplashScreen program displaying its About dialog. It is similar to the splash screen except it doesnt display a progress bar. It also displays an OK button so the user can close it and a rotated link to the support Web site www.vb-helper.com."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "A splash screen should not display system information such as memory usage or disk space. To manage system resources, the user should use the operating system tools not your application."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "When a user needs help with an application, the technical support people usually need to know the programs serial number. These numbers are often quite long (my Visual Basic serial number has 20 digits plus embedded dashes) so, to make entering the serial number in an e-mail easier, the About dialog displays it in a read-only text box. That allows the user to select the serial number, press Ctrl-C to copy it to the clipboard, and then press Ctrl-V to paste it into the e-mail."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "Many splash screens and About dialogs also contain hidden surprises, often called Easter Eggs. If you hover the mouse over a small area on the dot in the titles exclamation point, the mouse cursor changes to a crosshair. If you click that spot, the hidden screen shown in Figure 19-3 appears. Often this kind of hidden screen displays extra non-critical information such as the names of the developers who worked on the application."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "Some hidden screens are extremely elaborate. Some play music or make the developers names appear in an amusing animation. Ive even seen some that included complicated games that the user can play."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "While splash screens provide important information and feedback while the application is loading, hidden screens are definitely optional. Theyre usually fun and easy to build, however, so developers can add them in a spare moment without much wasted effort (complicated games notwithstanding)."))
        m_Paragraphs.Add(New ParagraphInfo(0, 20, 20, _
            "Displaying Splash Screens"))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "Visual Basic provides a way to automatically display a splash screen, although its a little awkward and not very well documented."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "First, build the form you want to use as a splash screen. Then open Solution Explorer and double-click the My Project item. On the Application tab, scroll to the bottom and select the form in the ""Splash screen"" dropdown as shown in Figure 19-4."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "At this point, Visual Basic will automatically display the splash screen when the application starts. It displays the form until the programs main form has finished its startup sequence or until a certain minimum amount of time has passed, whichever comes second. Unfortunately changing the minimum amount of time or adding data loading code requires some extra work on your part."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "To do either of these tasks, open Solution Explorer and double-click on the My Project entry. On the Application tab, click the View Application Events button at the bottom as shown in Figure 19-4."))
        m_Paragraphs.Add(New ParagraphInfo(2, 40, 20, _
            "' Override OnInitialize to set the minimum display time." & vbCrLf & _
            "Protected Overrides Function OnInitialize(ByVal commandLineArgs As _" & vbCrLf & _
            " System.Collections.ObjectModel.ReadOnlyCollection(Of String)) _" & vbCrLf & _
            " As Boolean" & vbCrLf & _
            "    ' Display the splash screen for at least 5 seconds." & vbCrLf & _
            "    Me.MinimumSplashScreenDisplayTime = 5000" & vbCrLf & _
            vbCrLf & _
            "    ' Continue initialization as usual." & vbCrLf & _
            "    Return MyBase.OnInitialize(commandLineArgs)" & vbCrLf & _
            "End Sub"))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "The minimum amount of time that a splash screen should be visible is a matter of preference but two or three seconds seems to work best. If the form vanishes after less than two seconds, the user doesnt really have a chance to see it. If the form sticks around for more than five seconds, it violates the ""five-second rule"" that says an interactive application should respond within five seconds whenever possible."))
        m_Paragraphs.Add(New ParagraphInfo(1, 0, 20, _
            "Visual Basic displays the splash screen for a minimum amount of time or until the main form finishes its startup sequence. To get the most benefit from the splash screen, you should make the application load data, connect to databases, use Web Services to check for updates, and perform other lengthy startup actions during this time."))

        ' Make the fonts we need.
        m_Fonts(0) = New Font("Times New Roman", 22, FontStyle.Bold, GraphicsUnit.Point)
        m_Fonts(1) = New Font("Times New Roman", 14, FontStyle.Regular, GraphicsUnit.Point)
        m_Fonts(2) = New Font("Courier New", 10, FontStyle.Regular, GraphicsUnit.Point)

        ' Set the next page number.
        m_NextPage = 1
    End Sub

    ' Clean up.
    Private Sub pdlgWrap_EndPrint(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintEventArgs) Handles pdocWrapped.EndPrint
        ' Dispose of the printing fonts.
        For i As Integer = 0 To 2
            m_Fonts(i).Dispose()
            m_Fonts(i) = Nothing
        Next i
    End Sub

    ' Change to True to draw margins and layout areas.
#Const DEBUGGING = False

    ' The PrintDocument needs a page. Print the next one.
    Private Sub pdlgWrap_PrintPage(ByVal sender As System.Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs) Handles pdocWrapped.PrintPage
#If DEBUGGING Then
        ' Draw the margins.
        e.Graphics.DrawRectangle(Pens.Red, e.MarginBounds)
#End If

        ' Print the page number.
        Dim x As Single = e.MarginBounds.Right + PAGE_NUM_MARGIN_X
        Dim y As Single = e.MarginBounds.Top - PAGE_NUM_MARGIN_Y
        Dim string_format As New StringFormat
        string_format.Alignment = StringAlignment.Center
        string_format.LineAlignment = StringAlignment.Center
        e.Graphics.DrawString(m_NextPage.ToString(), _
            m_Fonts(1), Brushes.Black, x, y, string_format)
        m_NextPage += 1

        ' Start at the top.
        y = e.MarginBounds.Top

        ' Loop while we have paragraphs.
        Do While m_Paragraphs.Count > 0
            ' Get the next piece of text we need to print.
            Dim para As ParagraphInfo = m_Paragraphs(0)
            Dim txt As String = para.Text

            ' See how much room we have.
            Dim layout_area As New SizeF( _
                e.MarginBounds.Width - para.Indent, _
                e.MarginBounds.Bottom - y)

            ' Make the height at least 1 to avoid confusing MeasureString
            If layout_area.Height < 1 Then layout_area.Height = 1

            ' See how much of the text will fit.
            Dim characters_fitted As Integer
            Dim lines_filled As Integer
            string_format.Alignment = StringAlignment.Near
            string_format.LineAlignment = StringAlignment.Near
            string_format.Trimming = StringTrimming.Word
            string_format.FormatFlags = StringFormatFlags.LineLimit
            Dim text_size As SizeF = e.Graphics.MeasureString(txt, _
                m_Fonts(para.FontNumber), layout_area, string_format, _
                characters_fitted, lines_filled)

            ' See if any characters fit.
            If characters_fitted > 0 Then
                ' Draw the text that fits.
                Dim layout_rect As New RectangleF( _
                    e.MarginBounds.Left + para.Indent, _
                    y, _
                    text_size.Width, _
                    text_size.Height)
                e.Graphics.DrawString(txt, m_Fonts(para.FontNumber), _
                    Brushes.Black, layout_rect, string_format)

#If DEBUGGING Then
                ' Draw a rectangle around the text.
                e.Graphics.DrawRectangle(Pens.Blue, Rectangle.Round(layout_rect))
#End If

                ' Increase y.
                y += text_size.Height + para.SpaceAfter
            End If

            ' See how much of this paragraph is left.
            para.Text = para.Text.Substring(characters_fitted)

            ' If all of this paragraph fit, remove it.
            If para.Text.Length < 1 Then m_Paragraphs.RemoveAt(0)

            ' If we have any text left in the paragraph,
            ' then we have filled this page.
            ' End this page and go to the next one.
            If para.Text.Length > 0 Then Exit Do
        Loop

        ' See if we have any text left to print.
        e.HasMorePages = (m_Paragraphs.Count > 0)
    End Sub
#End Region ' Printing Code
End Class
