Imports System.Reflection
Imports System.Collections.Generic

Public Class Form1
    ' Start with this program's executable selected.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        txtPath.Text = Application.ExecutablePath
        txtPath.Select(0, 0)
        dlgSelectAssembly.InitialDirectory = Application.ExecutablePath
    End Sub

    ' Let the user browse for an assembly.
    Private Sub btnBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnBrowse.Click
        If dlgSelectAssembly.ShowDialog(Me) = Windows.Forms.DialogResult.OK Then
            txtPath.Text = dlgSelectAssembly.FileName
        End If
    End Sub

    ' Display information about the assembly.
    Private Sub btnGo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGo.Click
        Dim assem As [Assembly] = [Assembly].LoadFrom(txtPath.Text)

        ' Display the assembly's full name.
        trvDetails.Nodes.Clear()
        trvDetails.Nodes.Add("Full Name: " & assem.FullName)
        trvDetails.Nodes.Add("Location: " & assem.Location)

        ' Display referenced assemblies.
        Dim ref_assem_node As TreeNode = trvDetails.Nodes.Add("Referenced Assemblies")
        For Each referenced_assembly As AssemblyName In assem.GetReferencedAssemblies()
            ref_assem_node.Nodes.Add(referenced_assembly.FullName)
        Next referenced_assembly

        ' Display the assembly's entry point.
        Dim entry_point As MethodInfo = assem.EntryPoint()
        trvDetails.Nodes.Add("Entry Point: " & entry_point.Name)

        ' Types.
        Dim types_node As TreeNode = trvDetails.Nodes.Add("Types")
        For Each exported_type As Type In assem.GetTypes()
            AddTypeData(types_node, exported_type)
        Next exported_type
    End Sub

    ' Return information about a type.
    Private Sub AddTypeData(ByVal parent_node As TreeNode, ByVal the_type As Type)
        Dim txt As String = ""
        If the_type.IsPublic Then txt &= "Public "
        If the_type.IsInterface Then
            ' Skip MustInherit for Interfaces.
            txt &= "Interface "
        Else
            If the_type.IsAbstract Then txt &= "MustInherit "
        End If
        If the_type.IsClass Then txt &= "Class "
        If the_type.IsEnum Then txt &= "Enum "
        If the_type.IsValueType Then txt &= "Structure "

        Dim type_name As String = the_type.Name
        If the_type.ContainsGenericParameters Then
            ' Remove whatever comes after `.
            type_name = type_name.Substring(0, type_name.IndexOf("`"))

            ' Compose the generic parameters.
            Dim generic_params As String = ""
            Dim generic_type As Type = the_type.GetGenericTypeDefinition
            For Each arg_type As Type In generic_type.GetGenericArguments()
                generic_params &= ", " & arg_type.Name
            Next arg_type
            If generic_params.Length > 0 Then generic_params = generic_params.Substring(2)
            type_name &= "(Of " & generic_params & ")"
        End If
        Dim type_node As TreeNode = parent_node.Nodes.Add(txt & type_name)

        If the_type.BaseType Is Nothing Then
            type_node.Nodes.Add("Inherits From: Nothing")
        Else
            type_node.Nodes.Add("Inherits From: " & the_type.BaseType.FullName)
        End If
        type_node.Nodes.Add("Attributes: " & the_type.Attributes.ToString)
        type_node.Nodes.Add("Qualified Name: " & the_type.AssemblyQualifiedName)
        '~ type_node.Nodes.Add("Contains Generic Parameters: " & the_type.ContainsGenericParameters)

        ' See if it's an enum.
        If the_type.IsEnum Then
            ' It's an enum. List its values.
            Dim values_node As TreeNode = type_node.Nodes.Add("Values")
            For Each field_info As FieldInfo In the_type.GetFields()
                If field_info.IsStatic Then
                    values_node.Nodes.Add(field_info.Name & " = " & CLng(field_info.GetValue(Nothing)))
                Else
                    values_node.Nodes.Add(field_info.Name & " (non static)")
                End If
            Next field_info
        Else
            ' It's not an enum.
            ' Interfaces.
            Dim ifaces() As Type = the_type.GetInterfaces()
            If ifaces.Length > 0 Then
                Dim ifaces_node As TreeNode = type_node.Nodes.Add("Interfaces")
                For Each iface_type As Type In ifaces
                    AddInterfaceData(ifaces_node, iface_type)
                Next iface_type
            End If

            ' Constructors.
            Dim constructor_infos() As ConstructorInfo = the_type.GetConstructors()

            ' If constructor_infos = Nothing,
            ' the type has only a default constructor.
            ' If constructor_infos is empty, it has no constructors.
            If constructor_infos Is Nothing Then
                ' It has no explicit constructors,
                ' only the default empty constructor.
                Dim constr_node As TreeNode = type_node.Nodes.Add("Constructors")
                constr_node.Nodes.Add("New " & type_name & "()")
            ElseIf constructor_infos.Length > 0 Then
                Dim constr_node As TreeNode = type_node.Nodes.Add("Constructors")
                For Each constructor_info As ConstructorInfo In constructor_infos
                    AddConstructorData(constr_node, type_name, constructor_info)
                Next constructor_info
            End If

            ' Fields.
            Dim field_infos() As FieldInfo = the_type.GetFields()
            If field_infos.Length > 0 Then
                Dim fields_node As TreeNode = type_node.Nodes.Add("Fields")
                For Each field_info As FieldInfo In field_infos
                    AddFieldData(fields_node, field_info)
                Next field_info
            End If

            ' Properties.
            Dim property_infos() As PropertyInfo = the_type.GetProperties()
            If property_infos.Length > 0 Then
                Dim properties_node As TreeNode = type_node.Nodes.Add("Properties")
                For Each property_info As PropertyInfo In property_infos
                    AddPropertyData(properties_node, property_info)
                Next property_info
            End If

            ' Methods.
            Dim method_infos() As MethodInfo = the_type.GetMethods()
            If method_infos.Length > 0 Then
                Dim methods_node As TreeNode = type_node.Nodes.Add("Methods")
                For Each method_info As MethodInfo In method_infos
                    AddMethodData(methods_node, method_info)
                Next method_info
            End If

            ' Events.
            Dim event_infos() As EventInfo = the_type.GetEvents()
            If event_infos.Length > 0 Then
                Dim events_node As TreeNode = type_node.Nodes.Add("Events")
                For Each event_info As EventInfo In event_infos
                    AddEventData(events_node, event_info)
                Next event_info
            End If

            ' Nested types.
            Dim nested_types() As Type = the_type.GetNestedTypes()
            If nested_types.Length > 0 Then
                Dim nested_types_node As TreeNode = type_node.Nodes.Add("Nested Types")
                For Each nested_type As Type In nested_types
                    AddTypeData(nested_types_node, nested_type)
                Next nested_type
            End If
        End If
    End Sub

    ' Return information about a type.
    Private Sub AddConstructorData(ByVal parent_node As TreeNode, ByVal type_name As String, ByVal constructor_info As ConstructorInfo)
        ' Get the constructor's parameters.
        Dim txt As String = "New " & type_name & _
            ParameterList(constructor_info.GetParameters())
        parent_node.Nodes.Add(txt)
    End Sub

    ' Return a string listing parameters from a collection.
    Private Function ParameterList(ByVal param_infos() As ParameterInfo) As String
        Dim params As String = ""
        For Each parameter_info As ParameterInfo In param_infos
            params &= ", " & GetParameterData(parameter_info)
        Next parameter_info
        If params.Length > 0 Then params = params.Substring(2)

        Return "(" & params & ")"
    End Function

    ' Return information about a parameter.
    Private Function GetParameterData(ByVal parameter_info As ParameterInfo) As String
        Dim txt As String = ""
        If parameter_info.IsOptional Then
            txt &= "Optional "
        End If
        If parameter_info.IsOut Then
            txt &= "ByRef "
        Else
            txt &= "ByVal "
        End If
        txt &= parameter_info.Name
        If parameter_info.ParameterType.IsArray Then
            txt &= "()"
            txt &= " As " & parameter_info.ParameterType.GetElementType.Name
        Else
            txt &= " As " & parameter_info.ParameterType.Name
        End If
        If parameter_info.IsOptional Then
            If parameter_info.ParameterType.Name = "String" Then
                txt &= " = """ & parameter_info.DefaultValue.ToString() & """"
            ElseIf parameter_info.DefaultValue Is Nothing Then
                txt &= " = Nothing"
            Else
                txt &= " = " & parameter_info.DefaultValue.ToString()
            End If
        End If

        Return txt
    End Function

    ' Add information about a field.
    Private Sub AddFieldData(ByVal fields_node As TreeNode, ByVal field_info As FieldInfo)
        Dim txt As String = ""
        If field_info.IsPublic Then txt &= "Public "
        If field_info.IsPrivate Then txt &= "Private "
        If field_info.IsFamily Then txt &= "Protected "
        If field_info.IsStatic Then txt &= "Shared "
        txt &= field_info.Name
        txt &= " As " & field_info.FieldType.Name
        fields_node.Nodes.Add(txt)
    End Sub

    ' Add information about a property.
    Private Sub AddPropertyData(ByVal properties_node As TreeNode, ByVal property_info As PropertyInfo)
        Dim txt As String = "Public "
        If property_info.CanRead AndAlso Not property_info.CanWrite Then
            txt &= "ReadOnly "
        ElseIf Not property_info.CanRead AndAlso property_info.CanWrite Then
            txt &= "WriteOnly "
        End If
        txt &= property_info.Name
        txt &= " As " & property_info.PropertyType.Name
        properties_node.Nodes.Add(txt)
    End Sub

    ' Add information about a method.
    Private Sub AddMethodData(ByVal methods_node As TreeNode, ByVal method_info As MethodInfo)
        methods_node.Nodes.Add(MethodSignature(method_info))
    End Sub

    ' Return a method's signature in a VB format.
    Private Function MethodSignature(ByVal method_info As MethodInfo) As String
        Dim txt As String = ""
        If method_info.IsFamilyAndAssembly Then
            txt &= "Protected Friend "
        ElseIf method_info.IsFamily Then
            txt &= "Protected "
        ElseIf method_info.IsFamilyOrAssembly Then
            txt &= "Friend "
        ElseIf method_info.IsPrivate Then
            txt &= "Private "
        ElseIf method_info.IsPublic Then
            txt &= "Public "
        End If
        If method_info.IsFinal Then txt &= "NotOverridable "
        If method_info.IsHideBySig Then txt &= "Overrides "
        If method_info.IsAbstract Then txt &= "MustOverride "
        If method_info.IsStatic Then txt &= "Shared "
        If method_info.ReturnType.Name = "Void" Then
            txt &= "Sub "
        Else
            txt &= "Function "
        End If
        txt &= method_info.Name
        txt &= ParameterList(method_info.GetParameters())
        If method_info.ReturnType.Name <> "Void" Then
            txt &= " As " & method_info.ReturnType.Name
        End If

        Return txt
    End Function

    ' Add information about an event.
    Private Sub AddEventData(ByVal events_node As TreeNode, ByVal event_info As EventInfo)
        events_node.Nodes.Add(event_info.Name)
    End Sub

    ' Add information about an implemented interface.
    Private Sub AddInterfaceData(ByVal ifaces_node As TreeNode, ByVal iface_type As Type)
        ifaces_node.Nodes.Add("Implements " & iface_type.Name)
    End Sub
End Class

Public Interface IAssignable
    Sub AssignToJob(ByVal job_name As String)
End Interface

Public MustInherit Class Person
    Public FirstName As String
    Private m_LastName As String
    Public Property LastName() As String
        Get
            Return m_LastName
        End Get
        Set(ByVal value As String)
            m_LastName = value
        End Set
    End Property

    Public Sub New()
    End Sub
    Public Sub New(ByVal new_FirstName As String, ByVal new_LastName As String)
        FirstName = new_FirstName
        LastName = new_LastName
    End Sub

    Protected MustOverride Sub ProtMustOverrideSub(ByVal i As Integer)
    Protected Friend MustOverride Sub ProtFriendMustOverrideSub(ByVal i As Integer)
    Public Overridable Sub OverrideMe(Optional ByVal lst As List(Of Boolean) = Nothing)
    End Sub
End Class

Public Class Employee
    Inherits Person
    Implements IAssignable

    Public EmployeeId As Integer

    Public Shared NumEmployees As Integer
    Private Shared PrivShared As String
    Protected Shared ProtShared As Integer
    Public Property Boss() As Person
        Get
            Return Nothing
        End Get
        Set(ByVal value As Person)
        End Set
    End Property

    Public ReadOnly Property ROProperty() As Integer
        Get
        End Get
    End Property
    Public WriteOnly Property WOProperty() As Integer
        Set(ByVal value As Integer)
        End Set
    End Property

    Public Sub New(ByVal new_FirstName As String, ByVal new_LastName As String)
        MyBase.New(new_FirstName, new_LastName)
    End Sub
    Public Sub New(ByVal names() As String)
        MyBase.New(names(0), names(1))
    End Sub
    Public Sub New(Optional ByVal new_EmployeeId As Integer = 0)
        EmployeeId = new_EmployeeId
    End Sub

    Public Function Promote(ByVal new_grade As Integer) As Integer
        Return 0
    End Function

    Public Function GetBonus() As Decimal
    End Function

    Public Event EmployeeFired()
    Private Event EmployeeRiffed()

    Public Sub AssignToJob(ByVal job_name As String) Implements IAssignable.AssignToJob
    End Sub

    Protected Friend Sub ProtFriendSub(ByVal i As Integer)
    End Sub
    Friend Sub FriendSub(ByVal i As Integer)
    End Sub

    Protected Overrides Sub ProtMustOverrideSub(ByVal i As Integer)
    End Sub
    Protected Friend NotOverridable Overrides Sub ProtFriendMustOverrideSub(ByVal i As Integer)
    End Sub

    Public Shared Sub SharedSub()
    End Sub

    Public Enum EmployeeType
        Serf
        Overseer
        Overlord
    End Enum

    Public Class Office
        Public Building As String
        Public OfficeNumber As Integer
        Public Phone As String
    End Class
End Class

Public Class Customer
    Inherits Person

    Public CustomerId As Integer

    Protected Overrides Sub ProtMustOverrideSub(ByVal i As Integer)
    End Sub
    Protected Friend NotOverridable Overrides Sub ProtFriendMustOverrideSub(ByVal i As Integer)
    End Sub
End Class

Public Class Manager
    Inherits Employee

    Public Department As String
End Class

Public Class BTreeNode(Of DataType)
    Public Sub New()
    End Sub
    Public Sub New(ByVal some_data As DataType)
    End Sub

End Class

Public Class DictionaryNode(Of KeyType, DataType)

End Class

Public Enum PersonType
    ptGeneric
    ptEmployee
    ptManager
    ptCustomer
End Enum

Public Structure AddressInfo
    Public Street As String
    Public City As String
    Public State As String
    Public Zip As String
    Public Sub New(ByVal new_Street As String, Optional ByVal new_City As String = "no city", Optional ByVal new_State As String = "no state", Optional ByVal new_Zip As String = "")
        Street = new_Street
        City = new_City
        State = new_State
        Zip = new_Zip
    End Sub
End Structure

Public Class TestClass(Of T1, T2)

End Class
