Imports System.Windows.Forms
Imports System.Drawing

' Identifies the tool installer class.
Public Interface IToolInstaller
    Sub InstallTools(ByVal tool_menu As ToolStripMenuItem, ByVal rch As RichTextBox)
End Interface

' Lists the available tool classes.
Public Class ToolInstaller
    Implements IToolInstaller

    ' Install tools for this application.
    Public Sub InstallTools(ByVal tool_menu As ToolStripMenuItem, ByVal rch As System.Windows.Forms.RichTextBox) Implements IToolInstaller.InstallTools
        Dim bold_toggler As New BoldStyleToggler
        bold_toggler.InstallTool(tool_menu, rch)

        Dim italic_toggler As New ItalicStyleToggler
        italic_toggler.InstallTool(tool_menu, rch)
    End Sub
End Class

Public Interface IRichTool
    ' Return the tool's menu caption.
    Sub InstallTool(ByVal tool_menu As ToolStripMenuItem, ByVal rch As System.Windows.Forms.RichTextBox)
End Interface

' Switches the selected text's Bold font property.
Public Class BoldStyleToggler
    Implements IRichTool

    ' A reference to our menu item.
    Private m_ToolStripMenuItem As ToolStripMenuItem

    ' A reference to our RichTextBox.
    Private m_RichTextBox As RichTextBox

    ' Install the tool.
    Public Sub InstallTool(ByVal tools_menu As ToolStripMenuItem, ByVal rch As System.Windows.Forms.RichTextBox) Implements IRichTool.InstallTool
        ' Create the Tool menu item.
        m_ToolStripMenuItem = New ToolStripMenuItem( _
            "&Bold", Nothing, AddressOf MenuItem_Click)
        m_ToolStripMenuItem.ShortcutKeys = Keys.Control Or Keys.B
        m_ToolStripMenuItem.Visible = True

        ' Add the item to the Tools menu.
        tools_menu.DropDownItems.Add(m_ToolStripMenuItem)

        ' Watch for the Tool menu opening.
        AddHandler tools_menu.DropDownOpening, AddressOf ToolsMenu_DropDownOpening

        ' Save a reference to the RichTextBox.
        m_RichTextBox = rch
    End Sub

    ' The user invoked the tool.
    Private Sub MenuItem_Click(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim tool_item As ToolStripMenuItem = DirectCast(sender, ToolStripMenuItem)

        ' Toggle the style.
        Dim the_style As FontStyle = m_RichTextBox.SelectionFont.Style
        Dim old_bold As Boolean = (the_style And FontStyle.Bold) = FontStyle.Bold
        If old_bold Then
            ' Make not bold.
            the_style = the_style And Not FontStyle.Bold
        Else
            ' Make bold.
            the_style = the_style Or FontStyle.Bold
        End If
        Dim new_style As FontStyle = the_style

        m_RichTextBox.SelectionFont = New Font(m_RichTextBox.SelectionFont, the_style)
    End Sub

    ' The user opened the Tools menu.
    ' Check or uncheck this menu item.
    Private Sub ToolsMenu_DropDownOpening(ByVal sender As Object, ByVal e As System.EventArgs)
        m_ToolStripMenuItem.Checked = _
                (m_RichTextBox.SelectionFont.Style And FontStyle.Bold) _
                    = FontStyle.Bold
    End Sub
End Class

' Switches the selected text's Italic font property.
Public Class ItalicStyleToggler
    Implements IRichTool

    ' A reference to our menu item.
    Private m_ToolStripMenuItem As ToolStripMenuItem

    ' A reference to our RichTextBox.
    Private m_RichTextBox As RichTextBox

    ' Install the tool.
    Public Sub InstallTool(ByVal tools_menu As ToolStripMenuItem, ByVal rch As System.Windows.Forms.RichTextBox) Implements IRichTool.InstallTool
        ' Create the Tool menu item.
        m_ToolStripMenuItem = New ToolStripMenuItem( _
            "&Italic", Nothing, AddressOf MenuItem_Click)
        m_ToolStripMenuItem.ShortcutKeys = Keys.Control Or Keys.I
        m_ToolStripMenuItem.Visible = True

        ' Add the item to the Tools menu.
        tools_menu.DropDownItems.Add(m_ToolStripMenuItem)

        ' Watch for the Tool menu opening.
        AddHandler tools_menu.DropDownOpening, AddressOf ToolsMenu_DropDownOpening

        ' Save a reference to the RichTextBox.
        m_RichTextBox = rch
    End Sub

    ' The user invoked the tool.
    Private Sub MenuItem_Click(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim tool_item As ToolStripMenuItem = DirectCast(sender, ToolStripMenuItem)

        ' Toggle the style.
        Dim the_style As FontStyle = m_RichTextBox.SelectionFont.Style
        Dim old_Italic As Boolean = (the_style And FontStyle.Italic) = FontStyle.Italic
        If old_Italic Then
            ' Make not Italic.
            the_style = the_style And Not FontStyle.Italic
        Else
            ' Make Italic.
            the_style = the_style Or FontStyle.Italic
        End If
        Dim new_style As FontStyle = the_style

        m_RichTextBox.SelectionFont = New Font(m_RichTextBox.SelectionFont, the_style)
    End Sub

    ' The user opened the Tools menu.
    ' Check or uncheck this menu item.
    Private Sub ToolsMenu_DropDownOpening(ByVal sender As Object, ByVal e As System.EventArgs)
        m_ToolStripMenuItem.Checked = _
                (m_RichTextBox.SelectionFont.Style And FontStyle.Italic) _
                    = FontStyle.Italic
    End Sub
End Class
